########
# ScoreProgress by Peter Caffin
# BSD style license.
# For more info, see http://pc.autons.net/stuff/scoreprogress/
########

package require Tk
package require snit
package provide ScoreProgress 1.00

snit::widgetadaptor ::scoreprogress::Init {
 ###############

 component tkcanvas
 component ttkentry

 delegate method * to tkcanvas except state
 delegate method state to ttkentry

 constructor {args} { 
  # Initial
  if {[info exists arg(-max)]} { set ::ScoreProgress_max($win) $arg(-max) } else { set ::ScoreProgress_max($win) 100 }
  if {![info exists arg(-width)]} { set arg(-width) 100 }
  if {![info exists arg(-height)]} { set ::ScoreProgress_height($win) 18 } else { set ::ScoreProgress_height($win) $arg(-height) }
  # Construct
  foreach {opt value} $args { set arg($opt) $value } 
  variable ttkentry
  ttk::frame $win
  ttk::entry ${win}_Entry
  installhull $win -padx 0 -pady 0 -sticky nsew
  set ttkentry ${win}_Entry
  install tkcanvas using ::canvas ${win}_Canvas -width $arg(-width) -height $::ScoreProgress_height($win) -bd 1 -relief sunken -highlightthickness 0
  if {[::tclmacbag::TileThemeNow] == "aqua"} { set pad 2 } else { set pad 1 }
  pack $ttkentry -in $win -padx 0 -pady 0 -expand true -fill both
  # Canvas
  pack $tkcanvas -in $ttkentry -padx 0 -pady 0 -expand true -fill both
  $tkcanvas create rectangle 0 0 0 0 -tags incomplete -outline #000000 -fill white  ;# White
  $tkcanvas create rectangle 0 0 0 0 -tags incorrect -outline #000000 -fill #C10E0E ;# Red
  $tkcanvas create rectangle 0 0 0 0 -tags correct   -outline #000000 -fill #5FB404 ;# Green
  $tkcanvas create rectangle 0 0 0 0 -tags completed -outline #000000 -fill #0055AA ;# Blue
  # Set current width (to test for resize later) and Rebind to last change (ie, none).
  set ::ScoreProgress_width($win) [winfo width $win]
  set ::ScoreProgress_lastupdate($win) "after 0"
  # Apply args
  foreach {opt value} $args {
   foreach o {-borderwidth -highlightthickness -background -bg -rmargin} { if {$opt eq $o} continue } 
   catch { $tkcanvas configure $opt $value }
   }
  bind $tkcanvas <Configure> [list $self updateState]
  }

 method updateState {} {

  }

 ###############
 }

interp alias {} ScoreProgress {} ::scoreprogress::Init

proc ScoreProgress_update {w args} {
 foreach {opt value} $args { set arg($opt) $value }
 set width [winfo width $w]
 # Reset maximum
 if {[info exists arg(-max)]} { set ::ScoreProgress_max($w) $arg(-max) }
 # Update progress bars
 if {[info exists arg(-completed)]} {
  set ratio [expr { 1.000000 * $arg(-completed) / $::ScoreProgress_max($w) }]
  set completed [expr $ratio*$width]
  if {$ratio >= 1.000} { set completed [expr $width-2] } ;# Keeps border
  # Display
  $w coords correct    0 0 0 0
  $w coords incorrect  0 0 0 0
  $w coords incomplete 0 0 0 0
  $w coords completed  0 0 $completed $::ScoreProgress_height($w)
  # Update Info
  set ::ScoreProgress_completed($w) $arg(-completed)
  } elseif {[info exists arg(-values)]} {
#  # Future option for progressive value display.
#  set incorrect [llength [lsearch -exact $arg(-values) 1]]
#  set correct [llength [lsearch -exact $arg(-values) 0]]
#  foreach t $arg(-values) {
#   }
  } else {
  if {![info exists arg(-correct)]} { set arg(-correct) 0 }
  if {![info exists arg(-incorrect)]} { set arg(-incorrect) 0 }
  if {![info exists arg(-incomplete)]} { set arg(-incomplete) 0 }
  # Incorrect
  set ratio [expr { 1.000000 * ($arg(-correct)+$arg(-incorrect)) / $::ScoreProgress_max($w) }]
  set incorrect [expr $ratio*$width]
  if {$ratio >= 1.000} { set incorrect [expr $width-2] }
  # Correct
  set ratio [expr { 1.000000 * $arg(-correct) / $::ScoreProgress_max($w) }]
  set correct [expr $ratio*$width]
  if {$ratio >= 1.000} { set correct [expr $width-2] }
  # Seen but incomplete
  set ratio [expr { 1.000000 * $arg(-incomplete) / $::ScoreProgress_max($w) }]
  set incomplete [expr $ratio*$width]
  if {$ratio >= 1.000} { set incomplete [expr $width-2] }
  # Display
  $w coords incorrect  1 1 $incorrect $::ScoreProgress_height($w)
  $w coords correct    1 1 $correct  $::ScoreProgress_height($w)
  $w coords incomplete 1 1 $incomplete $::ScoreProgress_height($w)
  $w coords completed  0 0 0 0
  # Update Info
  set ::ScoreProgress_correct($w) $arg(-correct)
  set ::ScoreProgress_incorrect($w) $arg(-incorrect)
  }
 update
 # Set current width (to test for resize later) and Rebind to last change.
 set ::ScoreProgress_width($w) [winfo width $w]
 set ::ScoreProgress_lastupdate($w) "ScoreProgress_update $w $args"
 bind $w <Configure> "ScoreProgress_resize $w"
 }

proc ScoreProgress_resize {w} {
 set width [winfo width $w]
 if {$width != $::ScoreProgress_width($w)} { eval $::ScoreProgress_lastupdate($w) }
 }
